/**
 * Copyright (c) 2008-2012 EBM WebSourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the New BSD License (3-clause license).
 *
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the New BSD License (3-clause license)
 * for more details.
 *
 * You should have received a copy of the New BSD License (3-clause license)
 * along with this program/library; If not, see http://directory.fsf.org/wiki/License:BSD_3Clause/
 * for the New BSD License (3-clause license).
 */
 
package org.ow2.easywsdl.extensions.wsdl4bpel.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.xml.namespace.QName;

import org.ow2.easywsdl.extensions.wsdl4bpel.api.PartnerLinkType;
import org.ow2.easywsdl.extensions.wsdl4bpel.api.Role;
import org.ow2.easywsdl.extensions.wsdl4bpel.api.WSDL4BPELException;
import org.ow2.easywsdl.extensions.wsdl4bpel.org.oasis_open.docs.wsbpel._2_0.plnktype.TPartnerLinkType;
import org.ow2.easywsdl.extensions.wsdl4bpel.org.oasis_open.docs.wsbpel._2_0.plnktype.TRole;
import org.ow2.easywsdl.wsdl.api.Description;
import org.ow2.easywsdl.wsdl.api.WSDLException;
import org.ow2.easywsdl.wsdl.api.abstractElmt.AbstractWSDLElementImpl;

/** 
* @author Nicolas Salatge - EBM WebSourcing
*/
public class PartnerLinkTypeImpl extends AbstractWSDLElementImpl<TPartnerLinkType> implements PartnerLinkType {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	
	private List<Role> roles = new ArrayList<Role>();
	
	private Description description = null;
	
    public PartnerLinkTypeImpl(final TPartnerLinkType plt, final Description desc) {
        super(plt, (AbstractWSDLElementImpl) desc);
        this.description = desc;
        
        for(TRole role: this.model.getRole()) {
        	roles.add(new RoleImpl(role, this));
        }
    }

	/**
	 * @return the desc
	 */
	public Description getDescription() {
		return description;
	}

	public void addRole(Role role) throws WSDL4BPELException {
		
		
		this.model.getRole().add((TRole) ((AbstractWSDLElementImpl)role).getModel());
		this.roles.add(role);

	}

	public List<Role> getRoles() throws WSDL4BPELException {
		return this.roles;
	}

	public List<Role> removeAllRoles() throws WSDL4BPELException {
        throw new UnsupportedOperationException();
	}

	public Role removeRole(Role plt) throws WSDL4BPELException {
        throw new UnsupportedOperationException();
	}

	public QName getQName() {
		return new QName(this.description.getTargetNamespace(), this.model.getName());
	}

	public void setQName(QName name) {
		this.model.setName(name.getLocalPart());
	}

	@Override
	public Map<QName, String> getOtherAttributes() throws WSDLException {
		return this.model.getOtherAttributes();
	}

	public Role getRole(String name) {
		Role res = null;
		for(Role role: this.roles) {
			if(role.getName().equals(name)) {
				res = role;
				break;
			}
		}
		return res;
	}
	
}
