/**
 * Copyright (c) 2008-2012 EBM WebSourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the New BSD License (3-clause license).
 *
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the New BSD License (3-clause license)
 * for more details.
 *
 * You should have received a copy of the New BSD License (3-clause license)
 * along with this program/library; If not, see http://directory.fsf.org/wiki/License:BSD_3Clause/
 * for the New BSD License (3-clause license).
 */
 
package org.ow2.easywsdl.extensions.wsdl4bpel.impl;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.xml.bind.JAXBElement;
import javax.xml.namespace.QName;

import org.ow2.easywsdl.extensions.wsdl4bpel.api.PartnerLinkType;
import org.ow2.easywsdl.extensions.wsdl4bpel.api.Property;
import org.ow2.easywsdl.extensions.wsdl4bpel.api.PropertyAlias;
import org.ow2.easywsdl.extensions.wsdl4bpel.api.WSDL4BPELElement;
import org.ow2.easywsdl.extensions.wsdl4bpel.api.WSDL4BPELException;
import org.ow2.easywsdl.extensions.wsdl4bpel.impl.inout.WSDL4BPELReaderImpl;
import org.ow2.easywsdl.extensions.wsdl4bpel.org.oasis_open.docs.wsbpel._2_0.plnktype.ObjectFactory;
import org.ow2.easywsdl.extensions.wsdl4bpel.org.oasis_open.docs.wsbpel._2_0.plnktype.TPartnerLinkType;
import org.ow2.easywsdl.wsdl.api.Description;
import org.ow2.easywsdl.wsdl.api.WSDLElement;
import org.ow2.easywsdl.wsdl.api.WSDLException;
import org.ow2.easywsdl.wsdl.api.abstractElmt.AbstractWSDLElementImpl;
import org.ow2.easywsdl.wsdl.org.w3.ns.wsdl.DescriptionType;
import org.ow2.easywsdl.wsdl.org.xmlsoap.schemas.wsdl.TDefinitions;
import org.ow2.easywsdl.wsdl.org.xmlsoap.schemas.wsdl.TExtensibleDocumented;
import org.w3c.dom.Element;

/**
 * Abstract super class for all WSDL Elements, providing some basic common functionality.
 * @author Nicolas Salatge - EBM WebSourcing
 */
public class WSDL4BPELElementImpl<T extends WSDLElement> implements WSDL4BPELElement {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	protected T tag;

	private List<PartnerLinkType> partnerLinkTypes = new ArrayList<PartnerLinkType>();

	private List<Property> properties = new ArrayList<Property>();

	private List<PropertyAlias> propertyAliases = new ArrayList<PropertyAlias>();
	
	private ObjectFactory factory = new ObjectFactory();

	public WSDL4BPELElementImpl(final T t) throws WSDL4BPELException {
		this.tag = t;
		try {
			getBPELDataInAnyDescription();
		} catch (WSDLException e) {
			throw new WSDL4BPELException(e);
		}
	}


	private void getBPELDataInAnyDescription() throws WSDL4BPELException,
	WSDLException {
		// get partner link in description
		List<JAXBElement> jaxbs = new ArrayList<JAXBElement>();
		JAXBElement jaxb = null;
		if(this.tag instanceof Description) {
			if(((AbstractWSDLElementImpl)this.tag).getModel() instanceof DescriptionType) {
				// wsdl 2.0
				Iterator<Object> it = ((DescriptionType)((AbstractWSDLElementImpl)this.tag).getModel()).getImportOrIncludeOrTypes().iterator();
				Object part = null;
				while (it.hasNext()) {
					part = it.next();
					// get partner link type
					jaxb = extractPartnerLinkInWSDL(part, DescriptionType.class);
					if(jaxb != null) {
						it = ((DescriptionType)((AbstractWSDLElementImpl)this.tag).getModel()).getImportOrIncludeOrTypes().iterator();
						jaxbs.add(jaxb);
					}

					// get property
					jaxb = extractPropertyInWSDL(part, DescriptionType.class);
					if(jaxb != null) {
						it = ((DescriptionType)((AbstractWSDLElementImpl)this.tag).getModel()).getImportOrIncludeOrTypes().iterator();
						jaxbs.add(jaxb);
					}

					// get property alias
					jaxb = extractPropertyAliasInWSDL(part, DescriptionType.class);
					if(jaxb != null) {
						it = ((DescriptionType)((AbstractWSDLElementImpl)this.tag).getModel()).getImportOrIncludeOrTypes().iterator();
						jaxbs.add(jaxb);
					}
				}
				
				for(JAXBElement jb: jaxbs) {
					((DescriptionType)((AbstractWSDLElementImpl)this.tag).getModel()).getImportOrIncludeOrTypes().add(jb);
				}
				
			} else if(((AbstractWSDLElementImpl)this.tag).getModel() instanceof TDefinitions) {
				// wsdl 1.1
				Iterator<Object> it = ((TExtensibleDocumented)((AbstractWSDLElementImpl)this.tag).getModel()).getAny().iterator();
				Object part = null;
				while (it.hasNext()) {
					part = it.next();
					// get partner link type
					jaxb = extractPartnerLinkInWSDL(part, TExtensibleDocumented.class);
					if(jaxb != null) {
						it = ((TExtensibleDocumented)((AbstractWSDLElementImpl)this.tag).getModel()).getAny().iterator();
						jaxbs.add(jaxb);
					}

					// get property
					jaxb = extractPropertyInWSDL(part, TExtensibleDocumented.class);
					if(jaxb != null) {
						it = ((TExtensibleDocumented)((AbstractWSDLElementImpl)this.tag).getModel()).getAny().iterator();
						jaxbs.add(jaxb);
					}

					// get property alias
					jaxb = extractPropertyAliasInWSDL(part, TExtensibleDocumented.class);
					if(jaxb != null) {
						it = ((TExtensibleDocumented)((AbstractWSDLElementImpl)this.tag).getModel()).getAny().iterator();
						jaxbs.add(jaxb);
					}
				}
				
				for(JAXBElement jb: jaxbs) {
					((TExtensibleDocumented)((AbstractWSDLElementImpl)this.tag).getModel()).getAny().add(jb);
				}
			} 
			
			
		}
	}


	private JAXBElement extractPartnerLinkInWSDL(Object part, Class type)
	throws WSDL4BPELException, WSDLException {
		JAXBElement res = null;
		if (part instanceof Element) {
			if((((Element)part).getLocalName().equals("partnerLinkType"))&&(((Element)part).getNamespaceURI().equals("http://docs.oasis-open.org/wsbpel/2.0/plnktype"))) {
				WSDL4BPELReaderImpl reader = new WSDL4BPELReaderImpl();
				JAXBElement<TPartnerLinkType> jaxbPlt = reader.convertElement2PartnerLinkType((Element)part);

				// change element by jaxb element
				if(type.getSimpleName().equals("TExtensibleDocumented")) {
					((TExtensibleDocumented)((AbstractWSDLElementImpl)this.tag).getModel()).getAny().remove(part);
					
					res = jaxbPlt;
				} else if(type.getSimpleName().equals("DescriptionType")) {
					((DescriptionType)((AbstractWSDLElementImpl)this.tag).getModel()).getImportOrIncludeOrTypes().remove(part);
					
					res = jaxbPlt;
				}
				
				// create partnerLinkType
				this.partnerLinkTypes.add(new PartnerLinkTypeImpl(jaxbPlt.getValue(), (Description) this.tag));
			}
		} else if((part instanceof JAXBElement)&&(((JAXBElement)part).getValue() instanceof TPartnerLinkType)) {
			// create partnerLinkType
			this.partnerLinkTypes.add(new PartnerLinkTypeImpl((TPartnerLinkType)((JAXBElement)part).getValue(), (Description) this.tag));
		} else if(part instanceof TPartnerLinkType) {
			// create partnerLinkType
			this.partnerLinkTypes.add(new PartnerLinkTypeImpl((TPartnerLinkType)part, (Description) this.tag));
		}
		return res;
	}

	private JAXBElement extractPropertyInWSDL(Object part, Class type)
	throws WSDL4BPELException, WSDLException {
		JAXBElement res = null;
		if (part instanceof Element) {
			if((((Element)part).getLocalName().equals("property"))&&(((Element)part).getNamespaceURI().equals("http://docs.oasis-open.org/wsbpel/2.0/varprop"))) {
				WSDL4BPELReaderImpl reader = new WSDL4BPELReaderImpl();
				JAXBElement<org.ow2.easywsdl.extensions.wsdl4bpel.org.oasis_open.docs.wsbpel._2_0.varprop.Property> jaxbPlt = reader.convertElement2Property((Element)part);

				// change element by jaxb element
				if(type.getSimpleName().equals("TExtensibleDocumented")) {
					((TExtensibleDocumented)((AbstractWSDLElementImpl)this.tag).getModel()).getAny().remove(part);
					res = jaxbPlt;
				} else if(type.getSimpleName().equals("DescriptionType")) {
					((DescriptionType)((AbstractWSDLElementImpl)this.tag).getModel()).getImportOrIncludeOrTypes().remove(part);
					res = jaxbPlt;
				}
				
				// create partnerLinkType
				this.properties.add(new PropertyImpl(jaxbPlt.getValue(), (Description) this.tag));
			}
		} else if((part instanceof JAXBElement)&&(((JAXBElement)part).getValue() instanceof org.ow2.easywsdl.extensions.wsdl4bpel.org.oasis_open.docs.wsbpel._2_0.varprop.Property)) {
			// create partnerLinkType
			this.properties.add(new PropertyImpl((org.ow2.easywsdl.extensions.wsdl4bpel.org.oasis_open.docs.wsbpel._2_0.varprop.Property)((JAXBElement)part).getValue(), (Description) this.tag));
		} else if(part instanceof org.ow2.easywsdl.extensions.wsdl4bpel.org.oasis_open.docs.wsbpel._2_0.varprop.Property) {
			// create partnerLinkType
			this.properties.add(new PropertyImpl((org.ow2.easywsdl.extensions.wsdl4bpel.org.oasis_open.docs.wsbpel._2_0.varprop.Property)part, (Description) this.tag));
		}
		return res;
	}


	private JAXBElement extractPropertyAliasInWSDL(Object part, Class type)
	throws WSDL4BPELException, WSDLException {
		JAXBElement res = null;
		if (part instanceof Element) {
			if((((Element)part).getLocalName().equals("propertyAlias"))&&(((Element)part).getNamespaceURI().equals("http://docs.oasis-open.org/wsbpel/2.0/varprop"))) {
				WSDL4BPELReaderImpl reader = new WSDL4BPELReaderImpl();
				JAXBElement<org.ow2.easywsdl.extensions.wsdl4bpel.org.oasis_open.docs.wsbpel._2_0.varprop.PropertyAlias> jaxbPlt = reader.convertElement2PropertyAlias((Element)part);

				// change element by jaxb element
				if(type.getSimpleName().equals("TExtensibleDocumented")) {
					((TExtensibleDocumented)((AbstractWSDLElementImpl)this.tag).getModel()).getAny().remove(part);
					res = jaxbPlt;
				} else if(type.getSimpleName().equals("DescriptionType")) {
					((DescriptionType)((AbstractWSDLElementImpl)this.tag).getModel()).getImportOrIncludeOrTypes().remove(part);
					res = jaxbPlt;
				}
				
				// create partnerLinkType
				this.propertyAliases.add(new PropertyAliasImpl(jaxbPlt.getValue(), (Description) this.tag));
			}
		} else if((part instanceof JAXBElement)&&(((JAXBElement)part).getValue() instanceof org.ow2.easywsdl.extensions.wsdl4bpel.org.oasis_open.docs.wsbpel._2_0.varprop.PropertyAlias)) {
			// create partnerLinkType
			this.propertyAliases.add(new PropertyAliasImpl((org.ow2.easywsdl.extensions.wsdl4bpel.org.oasis_open.docs.wsbpel._2_0.varprop.PropertyAlias)((JAXBElement)part).getValue(), (Description) this.tag));
		} else if(part instanceof org.ow2.easywsdl.extensions.wsdl4bpel.org.oasis_open.docs.wsbpel._2_0.varprop.PropertyAlias) {
			// create partnerLinkType
			this.propertyAliases.add(new PropertyAliasImpl((org.ow2.easywsdl.extensions.wsdl4bpel.org.oasis_open.docs.wsbpel._2_0.varprop.PropertyAlias)part, (Description) this.tag));
		}
		return res;
	}




	public List<PartnerLinkType> getPartnerLinkTypes() throws WSDL4BPELException {
		return this.partnerLinkTypes;
	}

	public void addPartnerLinkType(PartnerLinkType plt) throws WSDL4BPELException {
		this.partnerLinkTypes.add(plt);
		JAXBElement<TPartnerLinkType> pltModel = factory.createPartnerLinkType((TPartnerLinkType) ((AbstractWSDLElementImpl)plt).getModel());
		if(((AbstractWSDLElementImpl)this.tag).getModel() instanceof TDefinitions) {
			((TDefinitions)((AbstractWSDLElementImpl)this.tag).getModel()).getAny().add(pltModel);
		} else if(((AbstractWSDLElementImpl)this.tag).getModel() instanceof DescriptionType) {
			((DescriptionType)((AbstractWSDLElementImpl)this.tag).getModel()).getImportOrIncludeOrTypes().add(pltModel);
		} 
		
	}

	public PartnerLinkType removePartnerLinkType(PartnerLinkType plt) throws WSDL4BPELException {
        throw new UnsupportedOperationException();
	}

	public List<PartnerLinkType> removeAllPartnerLinkTypes() throws WSDL4BPELException {
        throw new UnsupportedOperationException();
	}


	public List<Property> getProperties() throws WSDL4BPELException {
		return this.properties;
	}


	public List<PropertyAlias> getAllPropertyAliases() throws WSDL4BPELException {
		return this.propertyAliases;
	}


	public Property getProperty(QName property) throws WSDL4BPELException {
		Property res = null; 
		for(Property prop: this.properties) {
			
			if(prop.getQName().equals(property)) {
				res = prop;
				break;
			}
		}
		return res;
	}


	public List<PropertyAlias> getPropertyAliases4ThisProperty(QName property)
			throws WSDL4BPELException {
		List<PropertyAlias> res = new ArrayList<PropertyAlias>(); 
		for(PropertyAlias prop: this.propertyAliases) {
			
			if(prop.getPropertyName().equals(property)) {
				res.add(prop);
			}
		}
		return res;
	}

	public PartnerLinkType getPartnerLinkType(QName plt) throws WSDL4BPELException {
		PartnerLinkType res = null;
		for(PartnerLinkType partner: this.partnerLinkTypes) {
			if(partner.getQName().equals(plt)) {
				res = partner;
				break;
			}
		}
		return res;
	}

}
