/**
 * Copyright (c) 2008-2012 EBM WebSourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the New BSD License (3-clause license).
 *
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the New BSD License (3-clause license)
 * for more details.
 *
 * You should have received a copy of the New BSD License (3-clause license)
 * along with this program/library; If not, see http://directory.fsf.org/wiki/License:BSD_3Clause/
 * for the New BSD License (3-clause license).
 */
 
package org.ow2.easywsdl.extensions.wsdl4bpel.impl.inout;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.Map;

import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.ow2.easywsdl.extensions.wsdl4bpel.WSDL4BPELFactory;
import org.ow2.easywsdl.extensions.wsdl4bpel.api.Description;
import org.ow2.easywsdl.extensions.wsdl4bpel.api.WSDL4BPELException;
import org.ow2.easywsdl.extensions.wsdl4bpel.api.WSDL4BPELReader;
import org.ow2.easywsdl.extensions.wsdl4bpel.impl.WSDL4BPELJAXBContext;
import org.ow2.easywsdl.extensions.wsdl4bpel.org.oasis_open.docs.wsbpel._2_0.plnktype.TPartnerLinkType;
import org.ow2.easywsdl.extensions.wsdl4bpel.org.oasis_open.docs.wsbpel._2_0.varprop.Property;
import org.ow2.easywsdl.extensions.wsdl4bpel.org.oasis_open.docs.wsbpel._2_0.varprop.PropertyAlias;
import org.ow2.easywsdl.wsdl.WSDLFactory;
import org.ow2.easywsdl.wsdl.api.WSDLException;
import org.ow2.easywsdl.wsdl.api.WSDLReader;
import org.ow2.easywsdl.wsdl.api.WSDLReader.FeatureConstants;
import org.ow2.easywsdl.wsdl.api.abstractElmt.AbstractWSDLReaderImpl;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.xml.sax.InputSource;

import com.ebmwebsourcing.easycommons.xml.Transformers;

/**
 * @author Nicolas Salatge - EBM WebSourcing
 */
public class WSDL4BPELReaderImpl implements WSDL4BPELReader {

	private WSDLReader reader = null;
	

	public WSDL4BPELReaderImpl() throws WSDL4BPELException {
		try {
			this.reader = WSDLFactory.newInstance().newWSDLReader();
		} catch (final WSDLException e) {
			throw new WSDL4BPELException(e);
		} 
	}

	public Object getFeature(final FeatureConstants name) {
		return this.reader.getFeature(name);
	}

	public Map<FeatureConstants, Object> getFeatures() {
		return this.reader.getFeatures();
	}

	public void setFeatures(final Map<FeatureConstants, Object> features) {
		((AbstractWSDLReaderImpl) this.reader).setFeatures(features);
	}

    /**
     * {@inheritDoc} 
     * 
     */
    public Description read(final URL wsdlURL) throws WSDL4BPELException, IOException, URISyntaxException {
        try {
            InputSource inputSource = new InputSource(wsdlURL.openStream());
            inputSource.setSystemId(wsdlURL.toString());

            return this.read(inputSource);
        } catch (final MalformedURLException e) {
            throw new RuntimeException("The provided well-formed URL has been detected as malformed !!");
        } catch (final WSDLException e) {
            throw new WSDL4BPELException(e);
        }
    }

    /**
     * {@inheritDoc} 
     * 
     */
    public Description read(final Document wsdlDocument) throws WSDL4BPELException, URISyntaxException {
        // The DOM Document needs to be converted into an InputStource
        final ByteArrayOutputStream baos = new ByteArrayOutputStream();
        final StreamResult streamResult = new StreamResult(baos);
        
        // FIXME: The Transformer creation is not thread-safe
        Transformer transformer = null;
        try {
            transformer = Transformers.takeTransformer();
            transformer.transform(new DOMSource(wsdlDocument), streamResult);
            baos.flush();
            baos.close();

            final InputSource documentInputSource = new InputSource(new ByteArrayInputStream(
                    baos.toByteArray()));
            documentInputSource.setSystemId(wsdlDocument.getBaseURI());

            return this.read(documentInputSource);
        } catch (final TransformerConfigurationException e) {
            throw new WSDL4BPELException(e);
        } catch (final TransformerException e) {
            throw new WSDL4BPELException(e);
        } catch (final IOException e) {
            throw new WSDL4BPELException(e);
        } finally {
            Transformers.releaseTransformer(transformer);
        }
    }

    /**
     * {@inheritDoc} 
     * 
     */
    public Description read(final InputSource wsdlInputSource) throws WSDL4BPELException, MalformedURLException, URISyntaxException {
        try {
            final org.ow2.easywsdl.wsdl.api.Description desc = this.reader.read(wsdlInputSource);
            return WSDL4BPELFactory.newInstance().addBPELElmt2Description(desc);
        } catch (final WSDLException e) {
            throw new WSDL4BPELException(e);
        }
    }

    public void setFeature(final FeatureConstants name, final Object value) throws WSDLException {
		this.reader.setFeature(name, value);
	}

	
    public JAXBElement<TPartnerLinkType> convertElement2PartnerLinkType(final Element pltElmt)
    throws WSDLException {
    	JAXBElement<TPartnerLinkType> res = null;
        try {
            res = WSDL4BPELJAXBContext.getJaxbContext().createUnmarshaller().unmarshal(pltElmt, TPartnerLinkType.class);
        } catch (final JAXBException e) {
            throw new WSDLException(
                    "Failed to build Java bindings from WSDL descriptor XML document", e);
        }
        return res;
    }
    
    public JAXBElement<Property> convertElement2Property(final Element property)
    throws WSDLException {
    	JAXBElement<Property> res = null;
        try {
            res = WSDL4BPELJAXBContext.getJaxbContext().createUnmarshaller().unmarshal(property, Property.class);
        } catch (final JAXBException e) {
            throw new WSDLException(
                    "Failed to build Java bindings from WSDL descriptor XML document", e);
        }
        return res;
    }
    
    public JAXBElement<PropertyAlias> convertElement2PropertyAlias(final Element property)
    throws WSDLException {
    	JAXBElement<PropertyAlias> res = null;
        try {
            res = WSDL4BPELJAXBContext.getJaxbContext().createUnmarshaller().unmarshal(property, PropertyAlias.class);
        } catch (final JAXBException e) {
            throw new WSDLException(
                    "Failed to build Java bindings from WSDL descriptor XML document", e);
        }
        return res;
    }
}
