/**
 * Copyright (c) 2008-2012 EBM WebSourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the New BSD License (3-clause license).
 *
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the New BSD License (3-clause license)
 * for more details.
 *
 * You should have received a copy of the New BSD License (3-clause license)
 * along with this program/library; If not, see http://directory.fsf.org/wiki/License:BSD_3Clause/
 * for the New BSD License (3-clause license).
 */
 
package org.ow2.easywsdl.extensions.wsdl4bpel.test;

import java.io.IOException;
import java.net.URISyntaxException;
import java.net.URL;

import javax.xml.namespace.QName;

import junit.framework.TestCase;

import org.junit.Assert;
import org.ow2.easywsdl.extensions.wsdl4bpel.WSDL4BPELFactory;
import org.ow2.easywsdl.extensions.wsdl4bpel.api.Description;
import org.ow2.easywsdl.extensions.wsdl4bpel.api.PartnerLinkType;
import org.ow2.easywsdl.extensions.wsdl4bpel.api.Role;
import org.ow2.easywsdl.extensions.wsdl4bpel.api.WSDL4BPELException;
import org.ow2.easywsdl.extensions.wsdl4bpel.impl.PartnerLinkTypeImpl;
import org.ow2.easywsdl.extensions.wsdl4bpel.impl.RoleImpl;
import org.ow2.easywsdl.extensions.wsdl4bpel.org.oasis_open.docs.wsbpel._2_0.plnktype.TPartnerLinkType;
import org.ow2.easywsdl.extensions.wsdl4bpel.org.oasis_open.docs.wsbpel._2_0.plnktype.TRole;
import org.w3c.dom.Document;

/** 
* @author Nicolas Salatge - EBM WebSourcing
*/
public class BPEL4WSDLDescriptorTest extends TestCase {

	public void testReaderPartnerLinkBPEL4WSDL11() throws URISyntaxException, IOException, WSDL4BPELException {

	    final URL wsdlURL = Thread.currentThread().getContextClassLoader().getResource("descriptors/processArtifacts.wsdl");
        final org.ow2.easywsdl.extensions.wsdl4bpel.api.Description bpel4wsdlDesc = WSDL4BPELFactory.newInstance()
                .newWSDLReader().read(wsdlURL);

		// assert on description
		Assert.assertNotNull(bpel4wsdlDesc);

		assertEquals(1, bpel4wsdlDesc.getPartnerLinkTypes().size());
		assertEquals(new QName("http://petals.ow2.org", "process"), bpel4wsdlDesc.getPartnerLinkTypes().get(0).getQName());
		assertEquals(1, bpel4wsdlDesc.getPartnerLinkTypes().get(0).getRoles().size());
		assertEquals("processProvider", bpel4wsdlDesc.getPartnerLinkTypes().get(0).getRoles().get(0).getName());
		assertEquals(new QName("http://petals.ow2.org", "process"), bpel4wsdlDesc.getPartnerLinkTypes().get(0).getRoles().get(0).getInterface().getQName());
	}
	
	public void testReaderPropertyBPEL4WSDL11() throws URISyntaxException, WSDL4BPELException, IOException {

	    final URL wsdlURL = Thread.currentThread().getContextClassLoader().getResource("descriptors/salesArtifacts.wsdl");
        final org.ow2.easywsdl.extensions.wsdl4bpel.api.Description bpel4wsdlDesc = WSDL4BPELFactory.newInstance()
                .newWSDLReader().read(wsdlURL);

		// assert on description
		Assert.assertNotNull(bpel4wsdlDesc);

		assertEquals(1, bpel4wsdlDesc.getProperties().size());
		assertEquals(2, bpel4wsdlDesc.getAllPropertyAliases().size());
	}


	public void testModifierBPEL4WSDL11() throws URISyntaxException, IOException, WSDL4BPELException {

	    final Document doc;
	    {
	        final URL wsdlURL = Thread.currentThread().getContextClassLoader().getResource("descriptors/processArtifacts.wsdl");
            final org.ow2.easywsdl.extensions.wsdl4bpel.api.Description bpel4wsdlDesc = WSDL4BPELFactory.newInstance()
                    .newWSDLReader().read(wsdlURL);
    
    		// assert on description
    		Assert.assertNotNull(bpel4wsdlDesc);
    
    		bpel4wsdlDesc.getPartnerLinkTypes().get(0).setQName(new QName("TOTO"));
    		bpel4wsdlDesc.getPartnerLinkTypes().get(0).getRoles().get(0).setName("TITI");
    
    		doc = WSDL4BPELFactory.newInstance().newWSDLWriter().getDocument(bpel4wsdlDesc);
    		doc.setDocumentURI(wsdlURL.toString());
        }
	    
	    {
            final org.ow2.easywsdl.extensions.wsdl4bpel.api.Description bpel4wsdlDesc = WSDL4BPELFactory.newInstance().newWSDLReader().read(doc);
        
        	Assert.assertNotNull(bpel4wsdlDesc);
        	assertEquals(new QName("http://petals.ow2.org", "TOTO"), bpel4wsdlDesc.getPartnerLinkTypes().get(0).getQName());
        	assertEquals("TITI", bpel4wsdlDesc.getPartnerLinkTypes().get(0).getRoles().get(0).getName());
	    }
	}


	public void testReaderBPEL4WSDL20() throws URISyntaxException, WSDL4BPELException, IOException {

	    final URL wsdlURL = Thread.currentThread().getContextClassLoader().getResource("descriptors/purchase.wsdl");
        final org.ow2.easywsdl.extensions.wsdl4bpel.api.Description bpel4wsdlDesc = WSDL4BPELFactory.newInstance()
                .newWSDLReader().read(wsdlURL);

		// assert on description
		Assert.assertNotNull(bpel4wsdlDesc);

		assertEquals(1, bpel4wsdlDesc.getPartnerLinkTypes().size());
		assertEquals(new QName("http://www.w3.org/2002/ws/sawsdl/spec/wsdl/order#", "orderProcess"), bpel4wsdlDesc.getPartnerLinkTypes().get(0).getQName());
		assertEquals(1, bpel4wsdlDesc.getPartnerLinkTypes().get(0).getRoles().size());
		assertEquals("orderProvider", bpel4wsdlDesc.getPartnerLinkTypes().get(0).getRoles().get(0).getName());
		assertEquals(new QName("http://www.w3.org/2002/ws/sawsdl/spec/wsdl/order#", "Order"), bpel4wsdlDesc.getPartnerLinkTypes().get(0).getRoles().get(0).getInterface().getQName());
	}


	public void testModifierBPEL4WSDL20() throws URISyntaxException, WSDL4BPELException, IOException {

	    final Document doc;
	    {
	        final URL wsdlURL = Thread.currentThread().getContextClassLoader().getResource("descriptors/purchase.wsdl");
    	    final org.ow2.easywsdl.extensions.wsdl4bpel.api.Description bpel4wsdlDesc = WSDL4BPELFactory.newInstance()
                    .newWSDLReader().read(wsdlURL);
    
            // assert on description
    		Assert.assertNotNull(bpel4wsdlDesc);
    
    		bpel4wsdlDesc.getPartnerLinkTypes().get(0).setQName(new QName("TOTO"));
    		bpel4wsdlDesc.getPartnerLinkTypes().get(0).getRoles().get(0).setName("TITI");
    
    		doc = WSDL4BPELFactory.newInstance().newWSDLWriter().getDocument(bpel4wsdlDesc);
	    }
	    
	    {
	        final org.ow2.easywsdl.extensions.wsdl4bpel.api.Description bpel4wsdlDesc =  WSDL4BPELFactory.newInstance()
	                .newWSDLReader().read(doc);

    		Assert.assertNotNull(bpel4wsdlDesc);
    		assertEquals(new QName("http://www.w3.org/2002/ws/sawsdl/spec/wsdl/order#", "TOTO"), bpel4wsdlDesc.getPartnerLinkTypes().get(0).getQName());
    		assertEquals("TITI", bpel4wsdlDesc.getPartnerLinkTypes().get(0).getRoles().get(0).getName());
	    }
	}

	public void testReaderAndWriterBPEL4WSDL11() throws  URISyntaxException, WSDL4BPELException, IOException {
		// WSDL 1.1
	    final URL wsdlURL = Thread.currentThread().getContextClassLoader().getResource("descriptors/processArtifacts.wsdl");
        final org.ow2.easywsdl.extensions.wsdl4bpel.api.Description bpel4wsdlDesc = WSDL4BPELFactory.newInstance()
                .newWSDLReader().read(wsdlURL);
		final String res = WSDL4BPELFactory.newInstance().newWSDLWriter().writeBPEL4WSDL(bpel4wsdlDesc);
		Assert.assertNotNull(res);

//		System.out.println(res);
	}

	public void testReaderAndWriterBPEL4WSDL20() throws  URISyntaxException, WSDL4BPELException, IOException {
		// WSDL 2.0
	    final URL wsdlURL = Thread.currentThread().getContextClassLoader().getResource("descriptors/purchase.wsdl");
        final org.ow2.easywsdl.extensions.wsdl4bpel.api.Description bpel4wsdlDesc = WSDL4BPELFactory.newInstance()
                .newWSDLReader().read(wsdlURL);
		final String res = WSDL4BPELFactory.newInstance().newWSDLWriter().writeBPEL4WSDL(bpel4wsdlDesc);
		Assert.assertNotNull(res);

//		System.out.println(res);
	}
	
	public void testAddPartnerLinkType() throws WSDL4BPELException, IOException, URISyntaxException{

        final Document doc;
        {
            final URL wsdlURL = Thread.currentThread().getContextClassLoader().getResource("descriptors/purchase.wsdl");
            final org.ow2.easywsdl.extensions.wsdl4bpel.api.Description desc = WSDL4BPELFactory.newInstance()
                    .newWSDLReader().read(wsdlURL);

            PartnerLinkType plt = new PartnerLinkTypeImpl(new TPartnerLinkType(), desc);
            plt.setQName(new QName("pltName"));
            Role role = new RoleImpl(new TRole(), plt);
            role.setName("roleName");
            role.setInterface(desc.getInterfaces().get(0));
            plt.addRole(role);
            desc.addPartnerLinkType(plt);

            doc = WSDL4BPELFactory.newInstance().newWSDLWriter().getDocument(desc);
        }
		
        {
            final Description desc = WSDL4BPELFactory.newInstance().newWSDLReader().read(doc);
            assertNotNull(desc);
            assertEquals(new QName("http://www.w3.org/2002/ws/sawsdl/spec/wsdl/order#", "pltName"), desc
                    .getPartnerLinkTypes().get(desc.getPartnerLinkTypes().size() - 1).getQName());
        }
	}
}
